@@ -21,7 +21,7 @@ class BiteOptimizer: public CBiteOptDeep {
 public:
 
     BiteOptimizer(long runid_, Fitness *fitfun_, int dim_, double *init_,
-            int seed_, int M_, int maxEvaluations_, double stopfitness_) {
+            int seed_, int M_, int stall_iterations_, int maxEvaluations_, double stopfitness_) {
         // runid used to identify a specific run
         runid = runid_;
         // fitness function to minimize
@@ -30,6 +30,8 @@ public:
         dim = dim_;
         // Depth to use, 1 for plain CBiteOpt algorithm, >1 for CBiteOptDeep. Expected range is [1; 36].
         M = M_ > 0 ? M_ : 1;
+        // terminate if stall_iterations iterations stalled, if <= 0 not used
+        stall_iterations = stall_iterations_;
         // maximal number of evaluations allowed.
         maxEvaluations = maxEvaluations_ > 0 ? maxEvaluations_ : 50000;
         // Number of iterations already performed.
@@ -93,7 +95,7 @@ public:
                 stop = 1;
                 break;
             }
-            if (stallCount > 64*dim) {
+            if (stall_iterations > 0 && stallCount > stall_iterations*dim) {
                 stop = 2;
                 break;
             }
@@ -104,6 +106,7 @@ private:
     long runid;
     Fitness *fitfun;
     int M; // deepness
+    int stall_iterations; // terminate if stall_iterations stalled, if <= 0 not used
     int dim;
     int maxEvaluations;
     double stopfitness;
@@ -122,7 +125,7 @@ using namespace biteopt;
 extern "C" {
 void optimizeBite_C(long runid, callback_type func, int dim, int seed,
         double *init, double *lower, double *upper, int maxEvals,
-        double stopfitness, int popsize, int M, double* res) {
+        double stopfitness, int M, int stall_iterations, double* res) {
     int n = dim;
     vec lower_limit(n), upper_limit(n);
     bool useLimit = false;
@@ -137,7 +140,7 @@ void optimizeBite_C(long runid, callback_type func, int dim, int seed,
         upper_limit.resize(0);
     }
     Fitness fitfun(func, n, 1, lower_limit, upper_limit);
-    BiteOptimizer opt(runid, &fitfun, dim, init, seed, M, maxEvals,
+    BiteOptimizer opt(runid, &fitfun, dim, init, seed, M, stall_iterations, maxEvals,
             stopfitness);
 
     try {
