@@ -206,8 +206,8 @@ public:
         for (iterations = 1;; iterations++) {
             // sort population
             ivec sindex = sort_index(nextY);
-            popY = nextY(sindex, Eigen::all);
-            popX = nextX(Eigen::all, sindex);
+            popY = nextY(sindex, Eigen::indexing::all);
+            popX = nextX(Eigen::indexing::all, sindex);
 
             bestX = popX.col(0);
             bestY = popY[0];
@@ -239,7 +239,7 @@ public:
                 //Produce the CR and F
                 double mu = 1
                         - sqrt(float(iterations / maxIter))
-                        * exp(float(-gen_stuck / iterations));
+                                * exp(float(-gen_stuck / iterations));
                 if (iterations % 2 == 1) {
                     CR = normreal(rs, 0.95, 0.01);
                     F = normreal(rs, mu, 1);
@@ -254,10 +254,10 @@ public:
                     if (j == jr || rnd01() < CR) {
                         if (r3 < popsize)
                             ui[j] = popX(j, r1)
-                            + F * (popX(j, r2) - popX(j, r3));
+                                    + F * (popX(j, r2) - popX(j, r3));
                         else
                             ui[j] = popX(j, r1)
-                            + F * ((popX)(j, r2) - sp[r3 - popsize][j]);
+                                    + F * ((popX)(j, r2) - sp[r3 - popsize][j]);
                         if (!fitfun->feasible(j, ui[j]))
                             ui[j] = fitfun->sample_i(j, *rs);
                     }
