@@ -3,17 +3,19 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory.
 
-__version__ = '1.3.14'
+__version__ = '1.3.29'
 
 __all__ = [
     'cmaes',
     'cmaescpp',
+    'crmfnes',
+    'crfmnescpp',
     'de',
     'dacpp',
     'decpp',
     'gcldecpp',
     'ldecpp',
-    'lgcldecpp',
+    'lcldecpp',
     'bitecpp',
     'csmacpp',
     'retry',
