@@ -48,10 +48,8 @@ def minimize(fun,
     ----------
     fun : callable
         The objective function to be minimized.
-            ``fun(x, *args) -> float``
-        where ``x`` is an 1-D array with shape (n,) and ``args``
-        is a tuple of the fixed parameters needed to completely
-        specify the function.
+            ``fun(x) -> float``
+        where ``x`` is an 1-D array with shape (n,)
     bounds : sequence or `Bounds`, optional
         Bounds on variables. There are two ways to specify the bounds:
             1. Instance of the `scipy.Bounds` class.
@@ -135,7 +133,8 @@ def minimize_plot(name, optimizer, fun, bounds, value_limit = math.inf,
     time0 = time.perf_counter() # optimization start time
     name += '_' + optimizer.name
     logger.info('optimize ' + name)       
-    store = Store(fun, bounds, capacity = 500, logger = logger, statistic_num = statistic_num)
+    store = Store(fun, bounds, capacity = 500, logger = logger, statistic_num = statistic_num, 
+                  num_retries=num_retries)
     ret = retry(store, optimizer.minimize, value_limit, workers, stop_fitness)
     impr = store.get_improvements()
     np.savez_compressed(name, ys=impr)
@@ -352,7 +351,7 @@ class Store(object):
     def add_result(self, y, xs, lower, upper, evals, limit=math.inf):
         """registers an optimization result at the store."""
         with self.add_mutex:
-            self.incr_count_evals(evals)
+            self.count_evals.value += evals
             if y < limit:
                 if y < self.best_y.value:
                     self.best_y.value = y
@@ -398,23 +397,18 @@ class Store(object):
         self.ys[pid] = y            
 
     def get_runs_compare_incr(self, limit):
+        """trigger sorting after check_interval calls. """
         with self.add_mutex:
             if self.count_runs.value < limit:
                 self.count_runs.value += 1
+                if self.count_runs.value % self.check_interval == self.check_interval-1:
+                    if self.eval_fac.value < self.max_eval_fac:
+                        self.eval_fac.value += self.eval_fac_incr
+                    self.sort()                
                 return True
             else:
                 return False 
 
-    def incr_count_evals(self, evals):
-        """registers the number of evaluations of an optimization run; 
-        trigger sorting after check_interval calls. """
-        if self.count_runs.value % self.check_interval == self.check_interval-1:
-            if self.eval_fac.value < self.max_eval_fac:
-                self.eval_fac.value += self.eval_fac_incr
-                #print(self.eval_fac.value)
-            self.sort()
-        self.count_evals.value += evals
-
     def dump(self):
         """logs the current status of the store if logger defined."""
         if self.logger is None:
