@@ -8,36 +8,30 @@ import math
 import os
 import ctypes as ct
 from scipy.optimize import Bounds
+from fcmaes.decpp import libcmalib
 
-basepath = os.path.dirname(os.path.abspath(__file__))
-if sys.platform.startswith('linux'):   
-    libgtoplib = ct.cdll.LoadLibrary(basepath + '/lib/libgtoplib.so')    
-elif 'mac' in sys.platform:
-    libgtoplib = ct.cdll.LoadLibrary(basepath + '/lib/libgtoplib.dylib')  
-else:
-    libgtoplib = ct.cdll.LoadLibrary(basepath + '/lib/libgtoplib.dll')
-
-freemem = libgtoplib.free_mem
-freemem.argtypes = [ct.POINTER(ct.c_double)]
-
-# for windows compatibility. Linux can pickle c pointers, windows can not
 astro_map = {  
-    "messengerfullC": libgtoplib.messengerfullC,
-    "messengerC": libgtoplib.messengerC,
-    "gtoc1C": libgtoplib.gtoc1C,
-    "cassini1C": libgtoplib.cassini1C,
-    "cassini1minlpC": libgtoplib.cassini1minlpC,
-    "cassini2C": libgtoplib.cassini2C,
-    "rosettaC": libgtoplib.rosettaC,
-    "sagasC": libgtoplib.sagasC,
-    "tandemC": libgtoplib.tandemC,
-    "tandemCu": libgtoplib.tandemCu
+    "messengerfullC": libcmalib.messengerfullC,
+    "messengerC": libcmalib.messengerC,
+    "gtoc1C": libcmalib.gtoc1C,
+    "cassini1C": libcmalib.cassini1C,
+    "cassini1minlpC": libcmalib.cassini1minlpC,
+    "cassini2C": libcmalib.cassini2C,
+    "rosettaC": libcmalib.rosettaC,
+    "sagasC": libcmalib.sagasC,
+    "tandemC": libcmalib.tandemC,
+    "tandemCu": libcmalib.tandemCu,
+    "cassini2minlpC": libcmalib.cassini2minlpC,
     }
 
+freemem = libcmalib.free_mem
+freemem.argtypes = [ct.POINTER(ct.c_double)]
+
 class Astrofun(object):
     """Provides access to ESAs GTOP optimization test functions."""
     def __init__(self, name, fun_c, lower, upper):    
         self.name = name 
+        self.fun_c = fun_c 
         self.bounds = Bounds(lower, upper)
         self.fun = python_fun(fun_c, self.bounds)
 
@@ -116,7 +110,7 @@ class Tandem(object):
     """ see https://www.esa.int/gsp/ACT/projects/gtop/tandem/ """
     def __init__(self, i, constrained=True):   
         self.name = ('Tandem ' if constrained else 'Tandem unconstrained ') + str(i+1)
-        self.cfun = "tandemC" if constrained else "tandemCu"
+        self.fun_c = "tandemC" if constrained else "tandemCu"
         self.fun = self.tandem
         self.bounds = Bounds([5475, 2.5, 0, 0, 20, 20, 20, 20, 0.01, 0.01, 0.01, 0.01, 1.05, 1.05, 1.05, -math.pi, -math.pi, -math.pi], 
                              [9132, 4.9, 1, 1, 2500, 2500, 2500, 2500, 0.99, 0.99, 0.99, 0.99, 10, 10, 10, math.pi,  math.pi,  math.pi])
@@ -131,7 +125,7 @@ class Tandem(object):
         n = len(x)
         array_type = ct.c_double * n   
         ints_type = ct.c_int * 5   
-        fun_c = astro_map[self.cfun]      
+        fun_c = astro_map[self.fun_c]      
         fun_c.argtypes = [ct.c_int, ct.POINTER(ct.c_double), ct.POINTER(ct.c_int)]
         try: # function is only defined inside bounds
             #x = np.asarray(x).clip(self.bounds.lb, self.bounds.ub)
@@ -146,7 +140,7 @@ class Tandem_minlp(object):
     """ see https://www.esa.int/gsp/ACT/projects/gtop/tandem/ """
     def __init__(self, constrained=True):   
         self.name = ('Tandem minlp ' if constrained else 'Tandem unconstrained minlp ') 
-        self.cfun = "tandemC" if constrained else "tandemCu"
+        self.fun_c = "tandemC" if constrained else "tandemCu"
         self.fun = self.tandem_minlp
         self.bounds = Bounds([5475, 2.5, 0, 0, 20, 20, 20, 20, 0.01, 0.01, 0.01, 0.01, 1.05, 1.05, 1.05, -math.pi, -math.pi, -math.pi,
                               1.51,1.51,1.51], 
@@ -159,7 +153,7 @@ class Tandem_minlp(object):
         seq = [3] + [int(round(xi)) for xi in xs[-3:]] + [6]
         array_type = ct.c_double * n   
         ints_type = ct.c_int * 5   
-        fun_c = astro_map[self.cfun]      
+        fun_c = astro_map[self.fun_c]      
         fun_c.argtypes = [ct.c_int, ct.POINTER(ct.c_double), ct.POINTER(ct.c_int)]
         try:
             val = fun_c(n, array_type(*x), ints_type(*seq))
@@ -189,18 +183,33 @@ class Cassini1multi(object):
 class Cassini1minlp(object):
     """ see https://www.esa.int/gsp/ACT/projects/gtop/cassini1/ """
     
-    def __init__(self, weights = [1,0,0,0]):    
-        Astrofun.__init__(self, 'Cassini1minlp', "Cassini1minlpC", 
-                           [-1000.,30.,100.,30.,400.,1000., 1.0,1.0,1.0,1.0 ],
-                           [0.,400.,470.,400.,2000.,6000., 9.0,9.0,9.0,9.0 ]       
+    def __init__(self, planets = [2,2,3,5]):    
+        Astrofun.__init__(self, 'Cassini1', "cassini1C", 
+                           [-1000.,30.,100.,30.,400.,1000.],
+                           [0.,400.,470.,400.,2000.,6000.]       
         )
-        self.fun = self.cassini1minlp
-        self.weights = weights
-        self.mfun = cassini1multi
-         
-    def cassini1minlp(self, x):
-        r = cassini1multi(x)
-        return self.weights[0]*r[0] + self.weights[1]*r[1] + self.weights[2]*r[2] + self.weights[3]*r[3]
+        self.fun = self.cassini1
+        self.planets = planets
+   
+    def cassini1(self, x):
+        return cassini1minlp(list(x) + self.planets)
+      
+def cassini1minlp(x):
+    n = len(x)
+    array_type = ct.c_double * n   
+    fun_c = astro_map["cassini1minlpC"]      
+    fun_c.argtypes = [ct.c_int, ct.POINTER(ct.c_double)]
+    fun_c.restype = ct.POINTER(ct.c_double)  
+    try: # function is only defined inside bounds
+        res = fun_c(n, array_type(*x))
+        dv = res[0]
+        freemem(res)
+        if not math.isfinite(dv):
+            dv = 1E10
+    except Exception as ex:
+        print(ex)
+        dv = 1E10
+    return dv
 
 def cassini1multi(x):
     n = len(x)
@@ -216,13 +225,46 @@ def cassini1multi(x):
         if not math.isfinite(dv):
             dv = 1E10
     except Exception as ex:
+        print(ex)
         dv = 1E10
         launch_dv = 1E10 
-    tof = x[1] + x[2] + x[3] + x[4]
+    tof = sum(x[1:6])
     launch_time = x[0] 
-    return [dv, tof, launch_time, launch_dv]
+    return [dv, tof, launch_time]
 
-  
+def cassini2multi(x):
+    n = len(x)
+    array_type = ct.c_double * n   
+    fun_c = astro_map["cassini2minlpC"]      
+    fun_c.argtypes = [ct.c_int, ct.POINTER(ct.c_double)]
+    fun_c.restype = ct.c_double  
+    try: # function is only defined inside bounds
+        dv = fun_c(n, array_type(*x))
+    except Exception as ex:
+        print(ex)
+        dv = 1E99
+    tof = sum(x[4:9])
+    launch_time = x[0] 
+    return [dv, tof, launch_time]
+ 
+class python_fun(object):
+    
+    def __init__(self, cfun, bounds):
+        self.cfun = cfun
+        self.bounds = bounds
+    
+    def __call__(self, x):
+        fun_c = astro_map[self.cfun]      
+        n = len(x)
+        array_type = ct.c_double * n   
+        try: # function is only defined inside bounds
+            # x = np.array(x).clip(self.bounds.lb, self.bounds.ub)
+            val = float(fun_c(n, array_type(*x)))
+            if not math.isfinite(val):
+                val = 1E10
+        except Exception as ex:
+            val = 1E10
+        return val 
 class python_fun(object):
     
     def __init__(self, cfun, bounds):
@@ -241,4 +283,4 @@ class python_fun(object):
         except Exception as ex:
             val = 1E10
         return val
-    
\ No newline at end of file
+    
