@@ -40,10 +40,8 @@ def minimize(fun,
     ----------
     fun : callable
         The objective function to be minimized.
-            ``fun(x, *args) -> float``
-        where ``x`` is an 1-D array with shape (dim,) and ``args``
-        is a tuple of the fixed parameters needed to completely
-        specify the function.
+            ``fun(x) -> float``
+        where ``x`` is an 1-D array with shape (dim,)
     bounds : sequence or `Bounds`, optional
         Bounds on variables. There are two ways to specify the bounds:
             1. Instance of the `scipy.Bounds` class.
@@ -70,7 +68,7 @@ def minimize(fun,
         If not workers is None, function evaluation is performed in parallel for the whole population. 
         Useful for costly objective functions but is deactivated for parallel retry.      
     normalize : boolean, optional
-        pheno -> if true geno transformation maps arguments to interval [-1,1] 
+        if true pheno -> geno transformation maps arguments to interval [-1,1] 
     update_gap : int, optional
         number of iterations without distribution update
            
