@@ -4,12 +4,9 @@
 # LICENSE file in the root directory.
 
 """ Numpy based implementation of multi objective
-    Differential Evolution using either DE/rand/1 or DE/best/1 strategy ('best' refers to the current pareto front'). 
-     
-    Can switch to NSGA-II like population update via parameter 'nsga_update'.
-    Then it works essentially like NSGA-II but instead of the tournament selection
-    the whole population is sorted and the best individuals survive. To do this
-    efficiently the crowd distance ordering is slightly inaccurate. 
+    Differential Evolution using either the DE/rand/1 strategy
+    or a NSGA-II like population update (parameter 'nsga_update=True)'.
+    Then it works similar to NSGA-II.
     
     Supports parallel fitness function evaluation. 
     
@@ -30,18 +27,26 @@
     For expensive objective functions (e.g. machine learning parameter optimization) use the workers
     parameter to parallelize objective function evaluation. The workers parameter is limited by the 
     population size.
+    
+    The ints parameter is a boolean array indicating which parameters are discrete integer values. This 
+    parameter was introduced after observing non optimal DE-results for the ESP2 benchmark problem: 
+    https://github.com/AlgTUDelft/ExpensiveOptimBenchmark/blob/master/expensiveoptimbenchmark/problems/DockerCFDBenchmark.py
+    If defined it causes a "special treatment" for discrete variables: They are rounded to the next integer value and
+    there is an additional mutation to avoid getting stuck at local minima. This behavior is specified by the internal
+    function _modifier which can be overwritten by providing the optional modifier argument. If modifier is defined,
+    ints is ignored. 
+    
+    See https://github.com/dietmarwo/fast-cma-es/blob/master/tutorials/MODE.adoc for a detailed description.
 """
 
 import numpy as np
-import os
-import time
+import os, sys, time
 import ctypes as ct
 from numpy.random import Generator, MT19937
 from fcmaes.evaluator import Evaluator
 from fcmaes import moretry
 import multiprocessing as mp
-from fcmaes.optimizer import logger
-from fcmaes import moretry
+from fcmaes.optimizer import logger, dtime
 
 os.environ['MKL_DEBUG_CPU_TYPE'] = '5'
 
@@ -56,9 +61,9 @@ def minimize(mofun,
              cr = 0.9, 
              nsga_update = False,
              pareto_update = 0,
+             ints = None,
+             modifier = None,
              rg = Generator(MT19937()),
-             logger = None,
-             plot_name = None,
              store = None):  
       
     """Minimization of a multi objjective function of one or more variables using
@@ -68,10 +73,8 @@ def minimize(mofun,
     ----------
     mofun : callable
         The objective function to be minimized.
-            ``mofun(x, *args) -> list(float)``
-        where ``x`` is an 1-D array with shape (n,) and ``args``
-        is a tuple of the fixed parameters needed to completely
-        specify the function.
+            ``mofun(x) -> list(float)``
+        where ``x`` is an 1-D array with shape (n,)
     nobj : int
         number of objectives
     ncon : int
@@ -98,17 +101,16 @@ def minimize(mofun,
     nsga_update = boolean, optional
         Use of NSGA-II or DE population update. Default is False    
     pareto_update = float, optional
-        Only applied if nsga_update = False. Use the pareto front for population update 
-        with probability pareto_update, else use the whole population. Default 0 - use always 
-        the whole population.      
+        Only applied if nsga_update = False. Favor better solutions for sample generation. Default 0 - 
+        use all population members with the same probability.   
+    ints = list or array, optional
+        indicating which parameters are discrete integer values. If defined these parameters will be
+        rounded to the next integer and some additional mutation of discrete parameters are performed.
+    modifier = callable, optional
+        used to overwrite the default behaviour induced by ints. If defined, the ints parameter is
+        ignored. Modifies all generated x vectors.
     rg = numpy.random.Generator, optional
         Random generator for creating random guesses.
-    logger : logger, optional
-        logger for log output for tell_one, If None, logging
-        is switched off. Default is a logger which logs both to stdout and
-        appends to a file ``optimizer.log``.
-    plot_name : plot_name, optional
-        if defined the pareto front is plotted during the optimization to monitor progress
     store : result store, optional
         if defined the optimization results are added to the result store. For multi threaded execution.
         use workers=1 if you call minimize from multiple threads
@@ -118,7 +120,7 @@ def minimize(mofun,
     x, y: list of argument vectors and corresponding value vectors of the optimization results. """
 
     mode = MODE(nobj, ncon, bounds, popsize, workers if not workers is None else 0, 
-            f, cr, nsga_update, pareto_update, rg, logger, plot_name)
+            f, cr, nsga_update, pareto_update, rg, ints, modifier)
     try:
         if workers and workers > 1:
             x, y, evals, iterations, stop = mode.do_optimize_delayed_update(mofun, max_evaluations, workers)
@@ -128,6 +130,7 @@ def minimize(mofun,
             store.add_results(x, y)
         return x, y
     except Exception as ex:
+        print(str(ex))  
         return None, None  
 
     
@@ -155,28 +158,43 @@ class store():
         self.num_stored = mp.RawValue(ct.c_int, 0) 
         self.num_added = mp.RawValue(ct.c_int, 0) 
 
+    def add_result(self, x, y):
+        with self.add_mutex:
+            self.num_added.value += 1
+            i = self.num_stored.value
+            if i < self.capacity:                
+                self.set_x(i, x) 
+                self.set_y(i, y)
+                i += 1
+            self.num_stored.value = i
+
     def add_results(self, xs, ys):
         with self.add_mutex:
             self.num_added.value += 1
             i = self.num_stored.value
             for j in range(len(xs)):
-                if i < self.capacity:                      
+                if i < self.capacity:                
                     self.set_x(i, xs[j]) 
                     self.set_y(i, ys[j])
                     i += 1
-                else: # store is full, replace with pareto front
-                    xf, yf = self.get_front()
-                    i = 0
-                    for k in range(len(xf)):                   
-                        self.set_x(i, xf[k]) 
-                        self.set_y(i, yf[k])
-                        i += 1
-                    if i == self.capacity:
-                        break
+                else:
+                    self.get_front(update=True)
+                    i = self.num_stored.value
+                    if i > 0.9*self.capacity: # give up
+                        return
             self.num_stored.value = i
                       
-    def get_front(self):
-        return moretry.pareto(self.get_xs(), self.get_ys())
+    def get_front(self, update=False):
+        stored = self.num_stored.value
+        xs = np.array([self.get_x(i) for i in range(stored)])
+        ys = np.array([self.get_y(i) for i in range(stored)])
+        xf, yf = moretry.pareto(xs, ys)
+        if update:
+            for i in range(len(yf)):                   
+                self.set_x(i, xf[i]) 
+                self.set_y(i, yf[i])
+            self.num_stored.value = len(yf)
+        return xf, yf
        
     def get_xs(self):
         return np.array([self.get_x(i) for i in range(self.num_stored.value)])
@@ -201,12 +219,14 @@ class MODE(object):
     
     def __init__(self, nobj, ncon, bounds, popsize = 64, workers = 0,
                  F = 0.5, Cr = 0.9, nsga_update = False, pareto_update = False, 
-                 rg = Generator(MT19937()), logger = None, plot_name = None):
+                 rg = Generator(MT19937()), ints = None, modifier = None):
         self.nobj = nobj
         self.ncon = ncon
         self.dim, self.lower, self.upper = _check_bounds(bounds, None)
         if popsize is None:
-            popsize = 31
+            popsize = 64
+        if popsize % 2 == 1 and nsga_update: # nsga update requires even popsize
+            popsize += 1
         self.popsize = popsize
         self.workers = workers 
         self.rg = rg
@@ -219,16 +239,20 @@ class MODE(object):
         self.evals = 0
         self.mutex = mp.Lock()
         self.p = 0
-        self.best_p = None
+        # nsga update doesn't support mixed integer
+        self.ints = None if nsga_update else np.array(ints)
+        # use default variable modifier for int variables if modifier is None
+        if modifier is None and not self.ints is None:
+            # adjust bounds because ints are rounded
+            self.lower = self.lower.astype(float)
+            self.upper = self.upper.astype(float)
+            self.lower[self.ints] -= .499999999
+            self.upper[self.ints] += .499999999
+            self.modifier = self._modifier
+        else:
+            self.modifier = modifier
         self._init()
-        if not logger is None:
-            self.logger = logger
-            self.n_evals = mp.RawValue(ct.c_long, 0)
-            self.time_0 = time.perf_counter()
-            if not plot_name is None:
-                self.plot_name = plot_name + '_mode_' + str(popsize) + '_' + \
-                    ('nsga_update' if nsga_update else ('de_update_' + str(pareto_update)))
-        
+                
     def ask(self):
         """ask for one new argument vector.
         
@@ -254,6 +278,8 @@ class MODE(object):
         Returns
         -------
         stop : int termination criteria, if != 0 loop should stop."""
+        if self._is_dominated(y, p):
+            return self.stop
         
         with self.mutex:  
             for dp in range(len(self.done)):
@@ -274,20 +300,6 @@ class MODE(object):
                         break
                     p += 1
                 self.pop_update()
-        
-        if hasattr(self, 'logger'):
-            self.n_evals.value += 1
-            if self.n_evals.value % 100000 == 99999:           
-                t = time.perf_counter() - self.time_0
-                c = self.n_evals.value
-                message = '"c/t={0:.2f} c={1:d} t={2:.2f} y={3!s} x={4!s}'.format(
-                    c/t, c, t, str(list(y)), str(list(x)))
-                self.logger.info(message)   
-                if hasattr(self, "plot_name") and not self.plot_name is None:           
-                    name = self.plot_name + ' '  + str(self.n_evals.value)
-                    np.savez_compressed(name, xs=self.x, ys=self.y) 
-                    moretry.plot(name, self.ncon, self.x, self.y)
-
         return self.stop 
 
     def _init(self):
@@ -316,7 +328,7 @@ class MODE(object):
                 self.x[self.popsize + p] = x
                 self.evals += 1
             self.pop_update()
-        x, y = filter(self.x, self.y)
+        x, y = _filter(self.x, self.y)
         return x, y, self.evals, self.iterations, self.stop
 
     def do_optimize_delayed_update(self, fun, max_evals, workers=mp.cpu_count()):
@@ -348,25 +360,29 @@ class MODE(object):
             self.evals += 1
             
         evaluator.stop()
-        x, y = filter(self.x, self.y)
+        x, y = _filter(self.x, self.y)
         return x, y, self.evals, self.iterations, self.stop
 
     def pop_update(self):
-        domination = pareto(self.y, self.nobj, self.ncon)
+        y0 = self.y
+        x0 = self.x
+        if self.nobj == 1:
+            yi = np.flip(np.argsort(self.y[:,0]))
+            y0 = self.y[yi]
+            x0 = self.x[yi]    
+        domination = pareto(y0, self.nobj, self.ncon)
         x = []
         y = []
         maxdom = int(max(domination))
         for dom in range(maxdom, -1, -1):
             domlevel = [p for p in range(len(domination)) if domination[p] == dom]
-            if dom == maxdom: # store pareto front in self.best_p
-                self.best_p =  domlevel
             if len(x) + len(domlevel) <= self.popsize:
                 # whole level fits
-                x = [*x, *self.x[domlevel]]
-                y = [*y, *self.y[domlevel]]
+                x = [*x, *x0[domlevel]]
+                y = [*y, *y0[domlevel]]
             else: # sort for crowding
-                nx = self.x[domlevel]
-                ny = self.y[domlevel]    
+                nx = x0[domlevel]
+                ny = y0[domlevel]    
                 si = [0]
                 if len(ny) > 1:                
                     cd = crowd_dist(ny)
@@ -393,17 +409,15 @@ class MODE(object):
             self.Cr = 0.5*self.Cr0 if self.iterations % 2 == 0 else self.Cr0
             self.F = 0.5*self.F0 if self.iterations % 2 == 0 else self.F0
         while True:
-            if self.rg.random() < self.pareto_update and (not self.best_p is None) and len(self.best_p) > 3: 
-                # sample from pareto front
-                rb = self.rg.integers(0, len(self.best_p))
-                rb = self.best_p[rb]
+            if self.pareto_update > 0: # sample elite solutions
                 r1, r2 = self.rg.integers(0, self.popsize, 2)
+                rb = int(self.popsize * (self.rg.random() ** (1.0 + self.pareto_update)))
             else:
                 # sample from whole population
                 r1, r2, rb = self.rg.integers(0, self.popsize, 3)
-                if r1 != p and r1 != rb and r1 != r2 and r2 != rb \
-                        and r2 != p and rb != p:
-                    break
+            if r1 != p and r1 != rb and r1 != r2 and r2 != rb \
+                and r2 != p and rb != p:
+                break
         xp = self.x[p]
         xb = self.x[rb]
         x1 = self.x[r1]
@@ -412,7 +426,9 @@ class MODE(object):
         r = self.rg.integers(0, self.dim)
         tr = np.array(
             [i != r and self.rg.random() > self.Cr for i in range(self.dim)])    
-        x[tr] = xp[tr]       
+        x[tr] = xp[tr]    
+        if not self.modifier is None:
+            x = self.modifier(x)   
         return x
     
     def _sample(self):
@@ -426,6 +442,27 @@ class MODE(object):
             return x
         else:
             return np.maximum(np.minimum(x, self.upper), self.lower)
+        
+    # default modifier for integer variables
+    def _modifier(self, x):
+        x_ints = x[self.ints]
+        n_ints = len(self.ints)
+        lb = self.lower[self.ints]
+        ub = self.upper[self.ints]
+        min_mutate = 0.5
+        max_mutate = max(1.0, n_ints/20.0)
+        to_mutate = self.rg.uniform(min_mutate, max_mutate)
+        # mututate some integer variables
+        x_ints = np.array([x if self.rg.random() > to_mutate/n_ints else 
+                           self.rg.uniform(lb[i], ub[i])
+                           for i, x in enumerate(x_ints)])
+        # round to int values
+        x[self.ints] = np.around(x_ints,0)
+        return x   
+    
+    def _is_dominated(self, y, p):
+        return np.all(np.array([y[i] >= self.y[p, i] for i in range(len(y))]))
+
                     
 def _check_bounds(bounds, dim):
     if bounds is None and dim is None:
@@ -435,7 +472,7 @@ def _check_bounds(bounds, dim):
     else:
         return len(bounds.ub), np.asarray(bounds.lb), np.asarray(bounds.ub)
  
-def filter(x, y):
+def _filter(x, y):
     ym = np.amax(y,axis=1)
     sorted = np.argsort(ym)
     x = x[sorted]
@@ -560,13 +597,90 @@ def variation(pop, lower, upper, rg, pro_c = 1, dis_c = 20, pro_m = 1, dis_m = 2
     offspring = np.maximum(np.minimum(offspring, upper), lower)
     return offspring
 
+
+def feasible(xs, ys, ncon, eps = 1E-2):
+    if ncon > 0: # select feasible
+        ycon = np.array([np.maximum(y[-ncon:], 0) for y in ys])  
+        con = np.sum(ycon, axis=1)
+        nobj = len(ys[0]) - ncon
+        feasible = np.array([i for i in range(len(ys)) if con[i] < eps])
+        if len(feasible) > 0:
+            xs, ys = xs[feasible], np.array([ y[:nobj] for y in ys[feasible]])
+        else:
+            print("no feasible")
+    return xs, ys
+
+def is_feasible(y, nobj, eps = 1E-2):
+    ncon = len(y) - nobj
+    if ncon == 0:
+        return True
+    else:
+        c = np.sum(np.maximum(y[-ncon:], 0))
+        return c < eps
+
+class wrapper(object):
+    """thread safe wrapper for objective function monitoring evaluation count and optimization result."""
+   
+    def __init__(self, fun, nobj, store = None, interval = 100000, plot = False, 
+                 name = None, logger=logger()):
+        self.fun = fun
+        self.nobj = nobj
+        self.n_evals = mp.RawValue(ct.c_long, 0)
+        self.time_0 = time.perf_counter()
+        self.best_y = mp.RawArray(ct.c_double, nobj)  
+        for i in range(nobj):
+            self.best_y[i] = sys.float_info.max
+        self.store = store
+        self.interval = interval
+        self.plot = plot
+        self.name = (fun.name if hasattr(fun, 'name') else 
+                     (fun.__name__ if hasattr(fun, '__name__') else fun.__class__.__name__)) \
+                        if name is None else name
+        self.logger = logger
+    
+    def __call__(self, x):
+        try:
+            y = self.fun(x)
+            if not self.store is None and is_feasible(y, self.nobj):
+                self.store.add_result(x, y[:self.nobj])
+            improve = False
+            for i in range(self.nobj):
+                if y[i] < self.best_y[i]:
+                    improve = True 
+                    self.best_y[i] = y[i] 
+            improve = improve and self.n_evals.value > 100
+            self.n_evals.value += 1
+            if self.n_evals.value % self.interval == 0 or improve:
+                constr = np.maximum(y[self.nobj:], 0) 
+                self.logger.info(
+                    str(dtime(self.time_0)) + ' ' + 
+                    str(self.n_evals.value) + ' ' + 
+                    str(round(self.n_evals.value/(1E-9 + dtime(self.time_0)),0)) + ' ' + 
+                    str(self.best_y[:]) + ' ' + str(list(constr)) + ' ' + str(list(x))) 
+                if not self.store is None:
+                    try:
+                        xs, ys = self.store.get_front(True)
+                        num = self.store.num_stored.value
+                        # self.logger.info(str(num) + ' ' + 
+                        #               ', '.join(['(' + ', '.join([str(round(yi,3)) for yi in y]) + ')' for y in ys]))
+                        name = self.name + '_' + str(num)
+                        np.savez_compressed(name, xs=xs, ys=ys)
+                        if self.plot:
+                            moretry.plot(name, 0, xs, ys, all=False)
+                    except Exception as ex:
+                        print(str(ex))                                                
+            return y
+        except Exception as ex:
+            print(str(ex))  
+            return None  
+        
+
 def minimize_plot(name, fun, nobj, ncon, bounds, popsize = 64, max_evaluations = 100000, nsga_update=False, 
-                  pareto_update=0, workers = mp.cpu_count(), logger=logger(), plot_name = None):
+                  pareto_update=0, workers = mp.cpu_count()):
     name += '_mode_' + str(popsize) + '_' + \
                 ('nsga_update' if nsga_update else ('de_update_' + str(pareto_update)))
     logger.info('optimize ' + name) 
     xs, ys = minimize(fun, nobj, ncon, bounds, popsize = popsize, max_evaluations = max_evaluations,
-                   nsga_update = nsga_update, pareto_update = pareto_update, workers=workers, 
-                   logger=logger, plot_name = plot_name)
+                   nsga_update = nsga_update, pareto_update = pareto_update, workers=workers)
     np.savez_compressed(name, xs=xs, ys=ys)
     moretry.plot(name, ncon, xs, ys)
