@@ -10,8 +10,10 @@ import sys
 import time
 import math
 import logging
+import ctypes as ct
+import multiprocessing as mp 
 
-from fcmaes import cmaes, cmaescpp, decpp, dacpp, hhcpp, gcldecpp, lcldecpp, ldecpp, csmacpp, bitecpp
+from fcmaes import crfmnes, crfmnescpp, cmaes, de, cmaescpp, decpp, dacpp, gcldecpp, lcldecpp, ldecpp, csmacpp, bitecpp
 
 _logger = None
 
@@ -62,6 +64,34 @@ def dtime(t0):
     """time since t0."""
     return round(time.perf_counter() - t0, 2)
 
+class wrapper(object):
+    """Fitness function wrapper for use with parallel retry."""
+
+    def __init__(self, fit, logger=logger()):
+        self.fit = fit
+        self.evals = mp.RawValue(ct.c_int, 0) 
+        self.best_y = mp.RawValue(ct.c_double, math.inf) 
+        self.t0 = time.perf_counter()
+        self.logger = logger
+
+    def __call__(self, x):
+        try:
+            self.evals.value += 1
+            y = self.fit(x)
+            y0 = y if np.isscalar(y) else sum(y)
+            if y0 < self.best_y.value:
+                self.best_y.value = y0
+                if not self.logger is None:
+                    self.logger.info(str(dtime(self.t0)) + ' '  + 
+                              str(self.evals.value) + ' ' + 
+                              str(round(self.evals.value/(1E-9 + dtime(self.t0)),0)) + ' ' + 
+                              str(self.best_y.value) + ' ' + 
+                              str(list(x)))
+            return y
+        except Exception as ex:
+            print(str(ex))  
+            return sys.float_info.max  
+    
 class Optimizer(object):
     """Provides different optimization methods for use with parallel retry."""
     
@@ -88,15 +118,19 @@ class Sequence(Optimizer):
             self.name += optimizer.name + ' -> '
             self.max_evaluations += optimizer.max_evaluations
         self.name = self.name[:-4]
-                  
+
     def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), store=None):
         evals = 0
+        y = math.inf
         for optimizer in self.optimizers:
             ret = optimizer.minimize(fun, bounds, guess, sdevs, rg, store)
-            guess = ret[0]
+            if ret[1] < y:
+                y = ret[1]
+                x = ret[0]
+            guess = x
             evals += ret[2]
-        return ret[0], ret[1], evals
-
+        return x, y, evals
+                  
 class Choice(Optimizer):
     """Random choice of optimizers."""
     
@@ -113,129 +147,202 @@ class Choice(Optimizer):
         opt = self.optimizers[choice]
         return opt.minimize(fun, bounds, guess, sdevs, rg, store)
 
-def de_cma(max_evaluations = 50000, popsize=31, stop_fittness = math.inf, 
+def de_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
            de_max_evals = None, cma_max_evals = None):
     """Sequence differential evolution -> CMA-ES."""
 
+    deEvals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(0.5*max_evaluations)
+        de_max_evals = int(deEvals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int(0.5*max_evaluations)
-    opt1 = De_cpp(max_evaluations = de_max_evals, stop_fittness = stop_fittness)
+        cma_max_evals = int((1.0-deEvals)*max_evaluations)
+    opt1 = De_cpp(popsize=popsize, max_evaluations = de_max_evals, stop_fitness = stop_fitness)
     opt2 = Cma_cpp(popsize=popsize, max_evaluations = cma_max_evals, 
-                   stop_fittness = stop_fittness)
+                   stop_fitness = stop_fitness)
     return Sequence([opt1, opt2])
 
-def de2_cma(max_evaluations = 50000, popsize=31, stop_fittness = math.inf, 
+def de_cma_py(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
+           de_max_evals = None, cma_max_evals = None):
+    """Sequence differential evolution -> CMA-ES in python."""
+
+    deEvals = np.random.uniform(0.1, 0.5)
+    if de_max_evals is None:
+        de_max_evals = int(deEvals*max_evaluations)
+    if cma_max_evals is None:
+        cma_max_evals = int((1.0-deEvals)*max_evaluations)
+    opt1 = De_python(popsize=popsize, max_evaluations = de_max_evals, stop_fitness = stop_fitness)
+    opt2 = Cma_python(popsize=popsize, max_evaluations = cma_max_evals, 
+                   stop_fitness = stop_fitness)
+    return Sequence([opt1, opt2])
+
+def de2_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
            de_max_evals = None, cma_max_evals = None):
     """Sequence differential evolution -> CMA-ES."""
 
+    deEvals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(0.5*max_evaluations)
+        de_max_evals = int(deEvals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int(0.5*max_evaluations)
+        cma_max_evals = int((1.0-deEvals)*max_evaluations)
     opt1 = Choice([GCLDE_cpp(de_max_evals), De_cpp(de_max_evals)])
-    opt2 = Cma_cpp(cma_max_evals, popsize=popsize, stop_fittness = stop_fittness)
+    opt2 = Cma_cpp(cma_max_evals, popsize=popsize, stop_fitness = stop_fitness)
     return Sequence([opt1, opt2])
 
-def de3_cma(max_evaluations = 50000, popsize=31, stop_fittness = math.inf, 
+def de3_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
            de_max_evals = None, cma_max_evals = None):
     """Sequence differential evolution -> CMA-ES."""
 
+    deEvals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(0.5*max_evaluations)
+        de_max_evals = int(deEvals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int(0.5*max_evaluations)
+        cma_max_evals = int((1.0-deEvals)*max_evaluations)
     opt1 =  Choice([GCLDE_cpp(de_max_evals), Cma_cpp(de_max_evals), De_cpp(de_max_evals)])
-    opt2 = Cma_cpp(cma_max_evals, popsize=popsize, stop_fittness = stop_fittness)
+    opt2 = Cma_cpp(cma_max_evals, popsize=popsize, stop_fitness = stop_fitness)
     return Sequence([opt1, opt2])
 
-def gclde_cma(max_evaluations = 50000, popsize=31, stop_fittness = math.inf, 
+def gclde_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
            de_max_evals = None, cma_max_evals = None, workers = None):
     """Sequence G-CL-differential evolution -> CMA-ES."""
 
+    deEvals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(2.0*max_evaluations/3.0)
+        de_max_evals = int(deEvals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int(max_evaluations/3.0)
-    opt1 = GCLDE_cpp(max_evaluations = de_max_evals, stop_fittness = stop_fittness, workers = workers)
+        cma_max_evals = int((1.0-deEvals)*max_evaluations)
+    opt1 = GCLDE_cpp(max_evaluations = de_max_evals, stop_fitness = stop_fitness, workers = workers)
     opt2 = Cma_cpp(popsize=popsize, max_evaluations = cma_max_evals, 
-                   stop_fittness = stop_fittness, workers = workers)
+                   stop_fitness = stop_fitness, workers = workers)
     return Sequence([opt1, opt2])
 
 def da_cma(max_evaluations = 50000, da_max_evals = None, cma_max_evals = None,
-           popsize=31, stop_fittness = math.inf):
+           popsize=31, stop_fitness = -math.inf):
     """Sequence differential evolution -> CMA-ES."""
 
+    daEvals = np.random.uniform(0.1, 0.5)
     if da_max_evals is None:
-        da_max_evals = int(0.5*max_evaluations)
+        da_max_evals = int(daEvals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int(0.5*max_evaluations)
-    opt1 = Da_cpp(max_evaluations = da_max_evals, stop_fittness = stop_fittness)
+        cma_max_evals = int((1.0-daEvals)*max_evaluations)
+    opt1 = Da_cpp(max_evaluations = da_max_evals, stop_fitness = stop_fitness)
     opt2 = Cma_cpp(popsize=popsize, max_evaluations = cma_max_evals, 
-                   stop_fittness = stop_fittness)
+                   stop_fitness = stop_fitness)
     return Sequence([opt1, opt2])
 
+class Crfmnes(Optimizer):
+    """CRFMNES Python implementation."""
+    
+    def __init__(self, max_evaluations=50000,
+                 popsize = 32, guess=None, stop_fitness = None,
+                 sdevs = None):        
+        Optimizer.__init__(self, max_evaluations, 'crfmnes')
+        self.popsize = popsize
+        self.stop_fitness = stop_fitness
+        self.guess = guess
+        self.sdevs = sdevs
+
+    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
+        ret = crfmnes.minimize(fun, bounds, 
+                self.guess if not self.guess is None else guess,
+                input_sigma = self.sdevs if not self.sdevs is None else sdevs,
+                max_evaluations = self.max_eval_num(store), 
+                popsize=self.popsize, 
+                stop_fitness = self.stop_fitness,
+                rg=rg, runid=self.get_count_runs(store))     
+        return ret.x, ret.fun, ret.nfev
+
+class Crfmnes_cpp(Optimizer):
+    """CRFMNES C++ implementation."""
+    
+    def __init__(self, max_evaluations=50000,
+                 popsize = 32, guess=None, stop_fitness = None,
+                 sdevs = None):        
+        Optimizer.__init__(self, max_evaluations, 'crfmnes cpp')
+        self.popsize = popsize
+        self.stop_fitness = stop_fitness
+        self.guess = guess
+        self.sdevs = sdevs
+
+    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
+        ret = crfmnescpp.minimize(fun, bounds, 
+                self.guess if not self.guess is None else guess,
+                input_sigma = self.sdevs if not self.sdevs is None else sdevs,
+                max_evaluations = self.max_eval_num(store), 
+                popsize=self.popsize, 
+                stop_fitness = self.stop_fitness,
+                rg=rg, runid=self.get_count_runs(store))     
+        return ret.x, ret.fun, ret.nfev
+
 class Cma_python(Optimizer):
     """CMA_ES Python implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fittness = None):        
+                 popsize = 31, guess=None, stop_fitness = None,
+                 update_gap = None, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'cma py')
         self.popsize = popsize
-        self.stop_fittness = stop_fittness
+        self.stop_fitness = stop_fitness
+        self.update_gap = update_gap
         self.guess = guess
+        self.sdevs = sdevs
 
     def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
         ret = cmaes.minimize(fun, bounds, 
-                self.guess if guess is None else guess,
-                input_sigma=sdevs, 
+                self.guess if not self.guess is None else guess,
+                input_sigma= self.sdevs if not self.sdevs is None else sdevs,
                 max_evaluations = self.max_eval_num(store), 
                 popsize=self.popsize, 
-                stop_fittness = self.stop_fittness,
-                rg=rg, runid=self.get_count_runs(store))     
+                stop_fitness = self.stop_fitness,
+                rg=rg, runid=self.get_count_runs(store),
+                update_gap = self.update_gap)     
         return ret.x, ret.fun, ret.nfev
 
 class Cma_cpp(Optimizer):
     """CMA_ES C++ implementation."""
    
     def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fittness = None, workers = None):        
+                 popsize = 31, guess=None, stop_fitness = None, 
+                 update_gap = None, workers = None, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'cma cpp')
         self.popsize = popsize
-        self.stop_fittness = stop_fittness
+        self.stop_fitness = stop_fitness
         self.guess = guess
+        self.sdevs = sdevs
+        self.update_gap = update_gap
         self.workers = workers
 
-    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), 
-                 store=None, workers = None):
-        ret = cmaescpp.minimize(fun, bounds, 
-                self.guess if guess is None else guess,
-                input_sigma=sdevs, 
-                max_evaluations = self.max_eval_num(store), 
-                popsize=self.popsize, 
-                stop_fittness = self.stop_fittness,
-                rg=rg, runid = self.get_count_runs(store), 
-                workers = self.workers if workers is None else workers)     
+    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()),
+                 store=None, workers=None):
+        ret = cmaescpp.minimize(fun, bounds,
+                self.guess if not self.guess is None else guess,
+                input_sigma= self.sdevs if not self.sdevs is None else sdevs,
+                max_evaluations=self.max_eval_num(store),
+                popsize=self.popsize,
+                stop_fitness=self.stop_fitness,
+                rg=rg, runid=self.get_count_runs(store),
+		              update_gap=self.update_gap,
+                workers=self.workers if workers is None else workers)     
         return ret.x, ret.fun, ret.nfev
 
 class Cma_orig(Optimizer):
     """CMA_ES original implementation."""
    
     def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fittness = None):        
+                 popsize = 31, guess=None, stop_fitness = None, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'cma orig')
         self.popsize = popsize
-        self.stop_fittness = stop_fittness
+        self.stop_fitness = stop_fitness
         self.guess = guess
+        self.sdevs = sdevs
 
     def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
         lower = bounds.lb
         upper = bounds.ub
-        guess = self.guess if guess is None else guess
+        guess = self.guess if not self.guess is None else guess
         if guess is None:
             guess = rg.uniform(lower, upper)
         max_evaluations = self.max_eval_num(store)   
+        input_sigma= self.sdevs if not self.sdevs is None else sdevs
         try:
             import cma
         except ImportError as e:
@@ -245,7 +352,7 @@ class Cma_orig(Optimizer):
                                                              'typical_x': guess,
                                                              'scaling_of_variables': scale(lower, upper),
                                                              'popsize': self.popsize,
-                                                             'CMA_stds': sdevs,
+                                                             'CMA_stds': input_sigma,
                                                              'verbose': -1,
                                                              'verb_disp': -1})
             evals = 0
@@ -265,42 +372,144 @@ class De_cpp(Optimizer):
     """Differential Evolution C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fittness = None, 
-                 keep = 200, f = 0.5, cr = 0.9):        
+                 popsize = None, stop_fitness = None, 
+                 keep = 200, f = 0.5, cr = 0.9, ints = None):        
         Optimizer.__init__(self, max_evaluations, 'de cpp')
         self.popsize = popsize
-        self.stop_fittness = stop_fittness
+        self.stop_fitness = stop_fitness
+        self.keep = keep
+        self.f = f
+        self.cr = cr
+        self.ints = ints
+
+    def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), store=None):
+        ret = decpp.minimize(fun, None, bounds, 
+                popsize=self.popsize, 
+                max_evaluations = self.max_eval_num(store), 
+                stop_fitness = self.stop_fitness,
+                keep = self.keep, f = self.f, cr = self.cr, ints=self.ints,
+                rg=rg, runid = self.get_count_runs(store))
+        return ret.x, ret.fun, ret.nfev
+
+class De_python(Optimizer):
+    """Differential Evolution Python implementation."""
+    
+    def __init__(self, max_evaluations=50000,
+                 popsize = None, stop_fitness = None, 
+                 keep = 200, f = 0.5, cr = 0.9, ints = None):        
+        Optimizer.__init__(self, max_evaluations, 'de py')
+        self.popsize = popsize
+        self.stop_fitness = stop_fitness
+        self.keep = keep
+        self.f = f
+        self.cr = cr
+        self.ints = ints
+
+    def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), store=None):
+        ret = de.minimize(fun, None, 
+                bounds, self.popsize, self.max_eval_num(store), workers=None,
+                stop_fitness = self.stop_fitness,
+                keep = self.keep, f = self.f, cr = self.cr, ints=self.ints,
+                rg=rg)
+        return ret.x, ret.fun, ret.nfev
+
+class Cma_ask_tell(Optimizer):
+    """CMA ask tell implementation."""
+    
+    def __init__(self, max_evaluations=50000,
+                 popsize = 31, guess=None, stop_fitness = None, sdevs = None):        
+        Optimizer.__init__(self, max_evaluations, 'cma at')
+        self.popsize = popsize
+        self.stop_fitness = stop_fitness
+        self.guess = guess
+        self.sdevs = sdevs
+
+    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
+        es = cmaes.Cmaes(bounds,
+                popsize = self.popsize, 
+                input_sigma = self.sdevs if not self.sdevs is None else sdevs, 
+                rg = rg)       
+        iters = self.max_eval_num(store) // self.popsize
+        evals = 0
+        for j in range(iters):
+            xs = es.ask()
+            ys = [fun(x) for x in xs]
+            evals += len(xs)
+            stop = es.tell(ys)
+            if stop != 0:
+                break 
+        return es.best_x, es.best_value, evals
+
+class De_ask_tell(Optimizer):
+    """Differential Evolution ask tell implementation."""
+    
+    def __init__(self, max_evaluations=50000,
+                 popsize = None, stop_fitness = None, 
+                 keep = 400, f = 0.5, cr = 0.9):        
+        Optimizer.__init__(self, max_evaluations, 'de at')
+        self.popsize = popsize
+        self.stop_fitness = stop_fitness
         self.keep = keep
         self.f = f
         self.cr = cr
 
     def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), store=None):
-        ret = decpp.minimize(fun, len(bounds.lb), bounds, 
-                popsize=self.popsize, 
-                max_evaluations = self.max_eval_num(store), 
-                stop_fittness = self.stop_fittness,
-                keep = self.keep, f = self.f, cr = self.cr,
-                rg=rg, runid = self.get_count_runs(store))
-        return ret.x, ret.fun, ret.nfev
+        dim = len(bounds.lb)
+        popsize = 31 if self.popsize is None else self.popsize
+        es = de.DE(dim, bounds, popsize = popsize, rg = rg, keep = self.keep, F = self.f, Cr = self.cr)  
+        es.fun = fun  #remove
+        max_evals = self.max_eval_num(store)
+        while es.evals < max_evals:
+            xs = es.ask()
+            ys = [fun(x) for x in xs]
+            stop = es.tell(ys, xs)
+            if stop != 0:
+                break 
+        return es.best_x, es.best_value, es.evals
+
+class random_search(Optimizer):
+    """Random search."""
+   
+    def __init__(self, max_evaluations=50000):        
+        Optimizer.__init__(self, max_evaluations, 'random')
+ 
+    def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), store=None):
+        dim, x_min, y_min = len(bounds.lb), None, None
+        max_chunk_size = 1 + 4e4 / dim
+        evals = self.max_eval_num(store)
+        budget = evals
+        while budget > 0:
+            chunk = int(max([1, min([budget, max_chunk_size])]))
+            X = rg.uniform(bounds.lb, bounds.ub, size = [chunk, dim])
+            F = [fun(x) for x in X]
+            index = np.argmin(F) if len(F) else None
+            if index is not None and (y_min is None or F[index] < y_min):
+                x_min, y_min = X[index], F[index]
+            budget -= chunk
+        return x_min, y_min, evals
 
 class LDe_cpp(Optimizer):
     """Local Differential Evolution C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fittness = None, 
-                 keep = 200, f = 0.5, cr = 0.9):        
+                 popsize = None, stop_fitness = None, 
+                 keep = 200, f = 0.5, cr = 0.9, guess = None, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'lde cpp')
         self.popsize = popsize
-        self.stop_fittness = stop_fittness
+        self.stop_fitness = stop_fitness
         self.keep = keep
         self.f = f
         self.cr = cr
+        self.guess = guess
+        self.sdevs = sdevs
 
     def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
-        ret = ldecpp.minimize(fun, bounds, guess, sdevs,
+        ret = ldecpp.minimize(fun, bounds, 
+                self.guess if not self.guess is None else guess, 
+                self.sdevs if not self.sdevs is None else sdevs,
                 popsize=self.popsize, 
                 max_evaluations = self.max_eval_num(store), 
-                stop_fittness = self.stop_fittness,
+                stop_fitness = self.stop_fitness,
                 keep = self.keep, f = self.f, cr = self.cr,
                 rg=rg, runid = self.get_count_runs(store))
         return ret.x, ret.fun, ret.nfev
@@ -309,11 +518,11 @@ class GCLDE_cpp(Optimizer):
     """GCL-Differential Evolution C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fittness = None, 
+                 popsize = None, stop_fitness = None, 
                  pbest = 0.7, f0 = 0.0, cr0 = 0.0, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'gclde cpp')
         self.popsize = popsize
-        self.stop_fittness = stop_fittness
+        self.stop_fitness = stop_fitness
         self.pbest = pbest
         self.f0 = f0
         self.cr0 = cr0
@@ -321,10 +530,10 @@ class GCLDE_cpp(Optimizer):
 
     def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), 
                  store=None, workers = None):
-        ret = gcldecpp.minimize(fun, bounds, 
+        ret = gcldecpp.minimize(fun, None, bounds, 
                 popsize=self.popsize, 
                 max_evaluations = self.max_eval_num(store), 
-                stop_fittness = self.stop_fittness,
+                stop_fitness = self.stop_fitness,
                 pbest = self.pbest, f0 = self.f0, cr0 = self.cr0,
                 rg=rg, runid = self.get_count_runs(store),
                 workers = self.workers if workers is None else workers)
@@ -334,22 +543,26 @@ class LCLDE_cpp(Optimizer):
     """LCL-Differential Evolution C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fittness = None, 
-                 pbest = 0.7, f0 = 0.0, cr0 = 0.0, workers = None):        
+                 popsize = None, stop_fitness = None, 
+                 pbest = 0.7, f0 = 0.0, cr0 = 0.0, workers = None, guess = None, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'lclde cpp')
         self.popsize = popsize
-        self.stop_fittness = stop_fittness
+        self.stop_fitness = stop_fitness
         self.pbest = pbest
         self.f0 = f0
         self.cr0 = cr0
         self.workers = workers
+        self.guess = guess
+        self.sdevs = sdevs
 
     def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), 
                  store=None, workers = None):
-        ret = lcldecpp.minimize(fun, bounds, guess, sdevs,
+        ret = lcldecpp.minimize(fun, bounds, 
+                self.guess if not self.guess is None else guess, 
+                self.sdevs if not self.sdevs is None else sdevs,
                 popsize=self.popsize, 
                 max_evaluations = self.max_eval_num(store), 
-                stop_fittness = self.stop_fittness,
+                stop_fitness = self.stop_fitness,
                 pbest = self.pbest, f0 = self.f0, cr0 = self.cr0,
                 rg=rg, runid = self.get_count_runs(store),
                 workers = self.workers if workers is None else workers)
@@ -360,9 +573,9 @@ class Da_cpp(Optimizer):
     """Dual Annealing C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 stop_fittness = None, use_local_search=True, guess = None):        
+                 stop_fitness = None, use_local_search=True, guess = None):        
         Optimizer.__init__(self, max_evaluations, 'da cpp',)
-        self.stop_fittness = stop_fittness
+        self.stop_fitness = stop_fitness
         self.use_local_search = use_local_search
         self.guess = guess
  
@@ -374,63 +587,48 @@ class Da_cpp(Optimizer):
                             rg=rg, runid = self.get_count_runs(store))
         return ret.x, ret.fun, ret.nfev
 
-class Hh_cpp(Optimizer):
-    """Harris hawks C++ implementation."""
-    
-    def __init__(self, max_evaluations=50000,
-                 popsize = 31, stop_fittness = None):        
-        Optimizer.__init__(self, max_evaluations, 'hh cpp')
-        self.popsize = popsize
-        self.stop_fittness = stop_fittness
-
-    def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), store=None):
-        ret = hhcpp.minimize(fun, len(bounds.lb), bounds, 
-                popsize=self.popsize, 
-                max_evaluations = self.max_eval_num(store), 
-                stop_fittness = self.stop_fittness,
-                rg=rg, runid = self.get_count_runs(store))
-        return ret.x, ret.fun, ret.nfev
-
 class Csma_cpp(Optimizer):
     """SCMA C++ implementation."""
    
     def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fittness = None, workers = None):        
+                 popsize = None, guess=None, stop_fitness = None, workers = None, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'scma cpp')
         self.popsize = popsize
-        self.stop_fittness = stop_fittness
+        self.stop_fitness = stop_fitness
         self.guess = guess
         self.workers = workers
+        self.sdevs = sdevs
 
     def minimize(self, fun, bounds, guess=None, sdevs=0.16, rg=Generator(MT19937()), 
                  store=None, workers = None):
         ret = csmacpp.minimize(fun, bounds, 
                 self.guess if guess is None else guess,
-                input_sigma=sdevs, 
+                self.sdevs if not self.sdevs is None else sdevs,
                 max_evaluations = self.max_eval_num(store), 
-                popsize=self.popsize, 
-                stop_fittness = self.stop_fittness,
+                stop_fitness = self.stop_fitness,
                 rg=rg, runid = self.get_count_runs(store))     
         return ret.x, ret.fun, ret.nfev
 
 class Bite_cpp(Optimizer):
     """Bite C++ implementation."""
    
-    def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fittness = None, workers = None):        
+    def __init__(self, max_evaluations=50000, 
+                 guess=None, stop_fitness = None, M = None, stall_iterations = None, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'bite cpp')
-        self.popsize = popsize
-        self.stop_fittness = stop_fittness
         self.guess = guess
+        self.stop_fitness = stop_fitness
+        self.M = 6 if M is None else M 
+        self.stall_iterations = 0 if stall_iterations is None else stall_iterations 
         self.workers = workers
 
     def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), 
                  store=None, workers = None):
+        
         ret = bitecpp.minimize(fun, bounds, 
                 self.guess if guess is None else guess,
                 max_evaluations = self.max_eval_num(store), 
-                popsize=self.popsize, 
-                stop_fittness = self.stop_fittness, M = 8,
+                stop_fitness = self.stop_fitness, M = self.M, 
+                stall_iterations = self.stall_iterations,
                 rg=rg, runid = self.get_count_runs(store))     
         return ret.x, ret.fun, ret.nfev
         
