@@ -43,10 +43,8 @@ def minimize(fun,
     ----------
     fun : callable
         The objective function to be minimized.
-            ``fun(x, *args) -> float``
-        where ``x`` is an 1-D array with shape (n,) and ``args``
-        is a tuple of the fixed parameters needed to completely
-        specify the function.
+            ``fun(x) -> float``
+        where ``x`` is an 1-D array with shape (n,)
     bounds : sequence or `Bounds`, optional
         Bounds on variables. There are two ways to specify the bounds:
             1. Instance of the `scipy.Bounds` class.
@@ -122,8 +120,13 @@ def minimize_plot(name, optimizer, fun, bounds, value_limit = math.inf,
     ret = retry(store, optimizer.minimize, num_retries, value_limit, workers, stop_fitness)
     impr = store.get_improvements()
     np.savez_compressed(name, ys=impr)
-    filtered = np.array([imp for imp in impr if imp[1] < plot_limit])
-    if len(filtered) > 0: impr = filtered
+    for _ in range(10):
+        filtered = np.array([imp for imp in impr if imp[1] < plot_limit])
+        if len(filtered) > 0: 
+            impr = filtered
+            break
+        else:
+            plot_limit *= 3
     logger.info(name + ' time ' + str(dtime(time0))) 
     plot(impr, 'progress_ret.' + name + '.png', label = name, 
          xlabel = 'time in sec', ylabel = r'$f$')
@@ -139,6 +142,11 @@ def plot(front, fname, interp=True, label=r'$\chi$',
             plot(front.T[np.array([0,i])].T, str(i) + '_' + fname, 
                  interp=interp, ylabel = r'$f_{0}$'.format(i+1))     
         return   
+    if len(front[0]) == 1:
+        ys = np.array(list(zip(range(100), [front[0][0]]*100)))
+        plot(ys, str(1) + '_' + fname, 
+                 interp=interp, xlabel = '', ylabel = r'$f_{0}$'.format(1))     
+        return      
     import matplotlib.pyplot as pl
     fig, ax = pl.subplots(1, 1)
     x = front[:, 0]; y = front[:, 1]
@@ -237,6 +245,7 @@ class Store(object):
             if not self.logger is None:
                 self.logger.info(str(self.time[si]) + ' '  + 
                           str(self.sevals.value) + ' ' + 
+                          str(int(self.sevals.value / self.time[si])) + ' ' + 
                           str(y) + ' ' + 
                           str(list(x)))
         return y
